/*!
 * @file MoveWheelChair.h
 *
 * @author Masaharu Shimizu@Future Robotics Technology Center, Chiba Institute of Technology
 *
 * @date 2008/07/16 林原先生作成のソースをlinuxへ移植
 * @date 2009/06/26 公開メンバーを変更　CalcOdometryをprivateに。またCalcOdometryのodometry引数を削除
 * @date 2010/06/29 @date 2010/06/29 CANインターフェースをInterface社IFCAN以外に、PeakSystem社PCAN,CANopenミドルウェアCanfestivalに対応
 *
 */
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <errno.h>

//CAN通信APIを下記より選択する
#define PCAN
//#define CANFESTIVAL
//#define IFCAN

#ifdef CANFESTIVAL
#include <canfestival.h>
#endif

#ifdef PCAN
#include <fcntl.h>    // O_RDWR
#include <libpcan.h>
#endif

#ifdef IFCAN
#include <ifcan.h>
#endif

/* Integers */
#define INTEGER8 int8_t
#define INTEGER16 int16_t
#define INTEGER24 int32_t
#define INTEGER32 int32_t
#define INTEGER40 int64_t
#define INTEGER48 int64_t
#define INTEGER56 int64_t
#define INTEGER64 int64_t

/* Unsigned integers */
#define UNS8   u_int8_t
#define UNS16  u_int16_t
#define UNS32  u_int32_t
#define UNS24  u_int32_t
#define UNS40  u_int64_t
#define UNS48  u_int64_t
#define UNS56  u_int64_t
#define UNS64  u_int64_t

/* Reals */
#define REAL32	float
#define REAL64 double

#define RECV_BUFF_SIZE 8

// CMoveWheelChair
/** CANインタフェースのモータードライバを駆動するクラス
 */
class CMoveWheelChair
{
public:
	CMoveWheelChair();
	virtual ~CMoveWheelChair();

	enum mode_T{
		LO_SPEED = 0,
		HI_SPEED
	};

	typedef struct {
		UNS16 cob_id;	/**< message's ID */
		UNS8 rtr;		/**< remote transmission request. (0 if not rtr message, 1 if rtr message) */
		UNS8 len;		/**< message's length (0 to 8) */
		UNS8 data[8]; /**< message's datas */
	} CanMessage;



	//int portHandle;
#ifdef PCAN
	HANDLE can_fd;
#endif

#ifdef CANFESTIVAL
	CAN_HANDLE can_fd;
#endif

#ifdef IFCAN
	int can_fd;
#endif

	int Init();
	int Close();


	int Move(float x, float y, float z);
	int GetEncoder(int enc[4]);

	int Stop();
	int Servo(int on);
	int Send(int id, int len, char *data);
	int Recv(unsigned long *msgCount, CanMessage* msg);

	int GetOdometry(float odo[3]);
	int SetOdometry(float odo[3]);
	int SetOdometryGain(double g_x, double g_y, double g_theta);

private:
	float odometry[3]; //!<0:x　サジタル、 1:y　ラテラル、 2:theta 左回り正
	int CalcOdometry(int enc[4], float st);
	double gain_theta; //!<オドメトリの姿勢角を補正するための係数
	double gain_x, gain_y; //!<オドメトリのx,y位置を補正するための係数
};
